;---------------------------------------------------------------
;       CheckMachineType
;       NON-PCI    =0
;       Wildcat    =1
;       i430,440   =2
;       i450KX     =3
;       RCC/NEC    =4
;---------------------------------------------------------------
CheckMachineType        PROC    near
                mov     ax,0B101h       ; PCI BIOS existence check
                int 1Ah
                jc      pci_check_fail
                cmp     EDX,20494350h   ; "PCI " signature check
                jne     pci_check_fail  ;
ExistHiddenPCI: mov     eax,PCIhostAddr        ; PCI-host bridge (excluding 450KX)
                mov     dx,0CF8h        ;
                out     dx,eax
                mov     dl,0FCh
                in      eax,dx
                cmp     ax,1004h        ; Is WildCat?
                je      Wilcat_type
                cmp     ax,1166h        ; Is RCC?
                je      RCC_type
                cmp     ax,8086h        ; Is Intel?
                je      Intel_type
                mov     eax,8000C800h   ; PCI25[00h] for 450KX
                mov     dx,0CF8h
                out     dx,eax
                mov     dx,0CFCh
                in      eax,dx
                cmp     eax,84C48086h   ; Device-Vendor ID of 450KX PCI-host
                je      i450KX_type
                mov     dx,offset msg_UKchip    ; Abort if Unkown chipset
                call    _puts
                jmp     DOSabend                ;

i450KX_type:    mov     MachineType,3
                call    SetP6MSR_F000
                mov     dx,offset msg_450KX
                ret
Intel_type:     mov     MachineType,2
                shr     eax,16
                cmp     ax,1250h
                je      Intel_430HX
                cmp     ax,1237h
                je      Intel_440FX
                cmp     ax,7030h
                je      Intel_430VX
                cmp     ax,7100h
                je      Intel_430TX
                cmp     ax,122Dh
                je      Intel_430FX
                mov     dx,offset msg_Intel
                ret
Intel_430HX:    mov     dx,offset msg_i430HX
                ret
Intel_430VX:    mov     dx,offset msg_i430VX
                ret
Intel_440FX:    call    SetP6MSR_F000
                mov     dx,offset msg_i440FX
                ret
Intel_430TX:    mov     dx,offset msg_i430TX
                ret
Intel_430FX:    mov     dx,offset msg_i430FX
                ret
RCC_type:       mov     MachineType,4
                mov     dx,offset msg_RCC
                ret
Wilcat_type:    mov     MachineType,1
                mov     dx,offset msg_Wildcat
                ret
pci_check_fail: mov     eax,PCIhostAddr
                mov     dx,0CF8h
                out     dx,eax
                mov     dl,0FCh
                in      eax,dx
                not     eax
                or      eax,eax
                jnz     ExistHiddenPCI
                mov     dx,offset msg_noPCI
                ret
PCIhostAddr     DD      80000000h
msg_UKchip      DB      0C1h,"Unknown chipset machine.",13,10,0
msg_noPCI       DB      0E1h,"No PCI machine.",13,10,0
msg_Wildcat     DB      0E1h,"VLSI Supercore 82C594 chipset machine.",13,10,0
msg_450KX       DB      0E1h,"PC-9821St20,St15",13,10,0
msg_RCC         DB      0E1h,"PC-9821RvII26,RsII26.",13,10,0
msg_Intel       DB      0E1h,"Standard Intel chipset machine.",13,10,0
msg_i440FX      DB      0E1h,"Intel 440FX chipset machine.",13,10,0
msg_i430HX      DB      0E1h,"Intel 430HX chipset machine.",13,10,0
msg_i430VX      DB      0E1h,"Intel 430VX chipset machine.",13,10,0
msg_i430TX      DB      0E1h,"Intel 430TX chipset machine.",13,10,0
msg_i430FX      DB      0E1h,"Intel 430FX chipset machine.",13,10,0
CheckMachineType  ENDP

;---------------------------------------------------------------
PCMC_PCI        DW      offset PCI_none
                DW      offset PCI_VLSI
                DW      offset PCI_Intel
                DW      offset PCI_450KX
                DW      offset PCI_RCC
PCMC_RAM        DW      offset RAM_none
                DW      offset RAM_VLSI
                DW      offset RAM_Intel
                DW      offset RAM_450KX
                DW      offset RAM_RCC
;---------------------------------------------------------------
SetP6MSR_F000   PROC    near
                xor     eax,eax
                inc     ax
                cpuid
                and     ax,0F00h     ; mask ODP,mode,step digits.
                cmp     ax,0600h     ; fam=P6
                jne     CPU_err
                pushf
                cli
                wbinvd
                mov     ecx,026Fh               ; fixMTRR F8000_FFFFF
                rdmsr
                mov     eax,MemAttribData       ; F8000-FBFFF
                mov     edx,MemAttribData       ; FC000-FFFFF
                wrmsr
                popf
                ret
MemAttribData   DD      04040404h               ; 06=WB, 04=WT
CPU_err:        mov     dx,offset err_CPUnotP6
                call    DOSabend
                ret
err_CPUnotP6    DB      041h,"CPU is not P6 family.",13,10,0
SetP6MSR_F000   ENDP
;---------------------------------------------------------------
PCI_Intel       PROC    near
                mov     dx,0CF8h
                mov     eax,PAM_Intel
                out     dx,eax
                mov     dx,0CFCh
                in      ax,dx
                and     ah,0Fh          ; 59h Mask higher 4bits
                out     dx,ax
                ret
PCI_Intel       ENDP
PAM_Intel       DD      80000058h       ; 59h-higher = F000
RAM_Intel       PROC    near
                mov     dx,0CF8h
                mov     eax,PAM_Intel
                out     dx,eax
                mov     dx,0CFCh
                in      ax,dx
                or      ah,70h          ; 59h-higher = F000
                out     dx,ax
                ret
RAM_Intel       ENDP
;---------------------------------------------------------------
PCI_VLSI        PROC    near
                mov     dx,0CF8h
                mov     eax,PAM_VLSI
                out     dx,eax
                mov     dx,0CFCh
                in      ax,dx
                mov     ah,00h          ; 69h
                out     dx,ax
                ret
PCI_VLSI        ENDP
PAM_VLSI        DD      80000068h       ; 69h = F000,F400,F800,FC00
RAM_VLSI        PROC    near
                mov     dx,0CF8h
                mov     eax,PAM_VLSI
                out     dx,eax
                mov     dx,0CFCh
                in      ax,dx
                mov     ah,0FFh         ; 69h
                out     dx,ax
                ret
RAM_VLSI        ENDP
;---------------------------------------------------------------
PCI_450KX       PROC    near
                mov     dx,0CF8h
                mov     eax,PAM_450KXMC
                out     dx,eax
                mov     dx,0CFCh
                in      ax,dx
                and     ah,00001111b
                out     dx,ax
                mov     dx,0CF8h
                mov     eax,PAM_450KXPB
                out     dx,eax
                mov     dx,0CFCh
                in      ax,dx
                or      ah,00110000b
                out     dx,ax
                ret
PCI_450KX       ENDP
PAM_450KXMC     DD      8000A058h       ; device #20
PAM_450KXPB     DD      8000C858h       ; device #25
RAM_450KX       PROC    near
                mov     dx,0CF8h
                mov     eax,PAM_450KXMC
                out     dx,eax
                mov     dx,0CFCh
                in      ax,dx
                or      ah,00000011b
                out     dx,ax
                mov     dx,0CF8h
                mov     eax,PAM_450KXPB
                out     dx,eax
                mov     dx,0CFCh
                in      ax,dx
                and     ah,11111100b
                out     dx,ax
                ret
RAM_450KX       ENDP
;---------------------------------------------------------------
;   Register meanings of PMC programmable attribute map  
;   RCC/NEC chipset (PC-9821RvII26)
;          RAM WE  RAM WE  RAM WE  RAM WE
; Reg.      7  6    5  4    3  2    1  0 bit(LSB)
;  73h      F800~   F000~   E800~   E000~
;  72h      DC00~   D800~   D400~   D000~
;  71h      CC00~   C800~   C400~   C000~
;  70h                   ?? A000-B000-
;---------------------------------------------------------------
PCI_RCC         PROC    near
                mov     dx,0CF8h
                mov     eax,PAM_RCC
                out     dx,eax
                mov     dx,0CFFh
                in      al,dx
                mov     RvMemStat,al
                and     al,03h
                out     dx,ax
                ret
PCI_RCC         ENDP
PAM_RCC         DD      80000070h
RAM_RCC         PROC    near
                mov     dx,0CF8h
                mov     eax,PAM_RCC
                out     dx,eax
                mov     dx,0CFFh
;;;             in      al,dx
                mov     al,RvMemStat
                out     dx,al
                ret
RvMemStat       DB      0
RAM_RCC         ENDP
;---------------------------------------------------------------
PCI_none:
RAM_none:       ret
